defmodule DingeWeb.DingLive.FormComponent do
  use DingeWeb, :live_component

  alias Dinge.Ledger

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        <%= @title %>
        <:subtitle>Use this form to manage ding records in your database.</:subtitle>
      </.header>

      <.simple_form
        for={@form}
        id="ding-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <.input field={@form[:title]} type="text" label="Title" />
        <.input field={@form[:count]} type="number" label="Count" value="0" />
        <.input field={@form[:target_count]} type="number" label="Target count" value="10" />
        <.input
          field={@form[:ding_type]}
          type="select"
          label="Ding type"
          prompt="Choose a value"
          options={Ecto.Enum.values(Dinge.Ledger.Ding, :ding_type)}
        />
        <.input
          field={@form[:target_type]}
          type="select"
          label="Target type"
          prompt="Choose a value"
          options={Ecto.Enum.values(Dinge.Ledger.Ding, :target_type)}
        />
        <.input
          field={@form[:status]}
          type="select"
          label="Status"
          prompt="Choose a value"
          options={Ecto.Enum.values(Dinge.Ledger.Ding, :status)}
        />
        <:actions>
          <.button phx-disable-with="Saving...">Save Ding</.button>
        </:actions>
      </.simple_form>
    </div>
    """
  end

  @impl true
  def update(%{ding: ding} = assigns, socket) do
    changeset = Ledger.change_ding(ding)

    {:ok,
     socket
     |> assign(assigns)
     |> assign_form(changeset)}
  end

  @impl true
  def handle_event("validate", %{"ding" => ding_params}, socket) do
    changeset =
      socket.assigns.ding
      |> Ledger.change_ding(ding_params)
      |> Map.put(:action, :validate)

    {:noreply, assign_form(socket, changeset)}
  end

  def handle_event("save", %{"ding" => ding_params}, socket) do
    save_ding(socket, socket.assigns.action, ding_params)
  end

  defp save_ding(socket, :edit, ding_params) do
    case Ledger.update_ding(socket.assigns.ding, ding_params) do
      {:ok, ding} ->
        notify_parent({:saved, ding})

        {:noreply,
         socket
         |> put_flash(:info, "Ding updated successfully")
         |> push_patch(to: socket.assigns.patch)}

      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign_form(socket, changeset)}
    end
  end

  defp save_ding(socket, :new, ding_params) do
    ding_params = Map.put(ding_params, "user_id", socket.assigns.user.id)

    case Ledger.create_ding(ding_params) do
      {:ok, ding} ->
        notify_parent({:saved, ding})

        {:noreply,
         socket
         |> put_flash(:info, "Ding created successfully")
         |> push_patch(to: socket.assigns.patch)}

      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign_form(socket, changeset)}
    end
  end

  defp assign_form(socket, %Ecto.Changeset{} = changeset) do
    assign(socket, :form, to_form(changeset))
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})
end
