defmodule DingeWeb.DingLive.Index do
  use DingeWeb, :live_view

  alias Dinge.Ledger
  alias Dinge.Ledger.Ding

  @impl true
  def mount(_params, _session, socket) do
    {:ok, stream(socket, :dings, Ledger.list_dings_by_user(socket.assigns.current_user))}
  end

  @impl true
  def handle_params(params, _url, socket) do
    {:noreply, apply_action(socket, socket.assigns.live_action, params)}
  end

  defp apply_action(socket, :edit, %{"id" => id}) do
    socket
    |> assign(:page_title, "Edit Ding")
    |> assign(:ding, Ledger.get_ding!(id))
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New Ding")
    |> assign(:ding, %Ding{})
  end

  defp apply_action(socket, :index, _params) do
    socket
    |> assign(:page_title, "Listing Dings")
    |> assign(:ding, nil)
  end

  @impl true
  def handle_info({DingeWeb.DingLive.FormComponent, {:saved, ding}}, socket) do
    {:noreply, stream_insert(socket, :dings, ding)}
  end

  @impl true
  def handle_event("delete", %{"id" => id}, socket) do
    ding = Ledger.get_ding!(id)
    {:ok, _} = Ledger.delete_ding(ding)

    {:noreply, stream_delete(socket, :dings, ding)}
  end
end
