defmodule Dinge.LedgerTest do
  use Dinge.DataCase

  alias Dinge.Ledger

  describe "dings" do
    alias Dinge.Ledger.Ding

    import Dinge.LedgerFixtures

    @invalid_attrs %{count: nil, status: nil, title: nil, target_count: nil, ding_type: nil, target_type: nil}

    test "list_dings/0 returns all dings" do
      ding = ding_fixture()
      assert Ledger.list_dings() == [ding]
    end

    test "get_ding!/1 returns the ding with given id" do
      ding = ding_fixture()
      assert Ledger.get_ding!(ding.id) == ding
    end

    test "create_ding/1 with valid data creates a ding" do
      valid_attrs = %{count: 42, status: :active, title: "some title", target_count: 42, ding_type: :countdown, target_type: :default}

      assert {:ok, %Ding{} = ding} = Ledger.create_ding(valid_attrs)
      assert ding.count == 42
      assert ding.status == :active
      assert ding.title == "some title"
      assert ding.target_count == 42
      assert ding.ding_type == :countdown
      assert ding.target_type == :default
    end

    test "create_ding/1 with invalid data returns error changeset" do
      assert {:error, %Ecto.Changeset{}} = Ledger.create_ding(@invalid_attrs)
    end

    test "update_ding/2 with valid data updates the ding" do
      ding = ding_fixture()
      update_attrs = %{count: 43, status: :inactive, title: "some updated title", target_count: 43, ding_type: :countup, target_type: :daily}

      assert {:ok, %Ding{} = ding} = Ledger.update_ding(ding, update_attrs)
      assert ding.count == 43
      assert ding.status == :inactive
      assert ding.title == "some updated title"
      assert ding.target_count == 43
      assert ding.ding_type == :countup
      assert ding.target_type == :daily
    end

    test "update_ding/2 with invalid data returns error changeset" do
      ding = ding_fixture()
      assert {:error, %Ecto.Changeset{}} = Ledger.update_ding(ding, @invalid_attrs)
      assert ding == Ledger.get_ding!(ding.id)
    end

    test "delete_ding/1 deletes the ding" do
      ding = ding_fixture()
      assert {:ok, %Ding{}} = Ledger.delete_ding(ding)
      assert_raise Ecto.NoResultsError, fn -> Ledger.get_ding!(ding.id) end
    end

    test "change_ding/1 returns a ding changeset" do
      ding = ding_fixture()
      assert %Ecto.Changeset{} = Ledger.change_ding(ding)
    end
  end
end
